package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.sort.DummySort;
import de.uni_marburg.powersort.sort.MergeSort;
import de.uni_marburg.powersort.sort.TimSort;
import de.uni_marburg.powersort.data.DescendingIntegers;
import de.uni_marburg.powersort.data.RandomIntegers;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final SortImpl[] sortImplementations = getSortImplementations();
        final List<Supplier<ObjectSupplier>> sortInputSuppliers = getSortInputSuppliers();

        for (Supplier<ObjectSupplier> sortInputSupplier : sortInputSuppliers) {
            ObjectSupplier objectSupplier = sortInputSupplier.get();
            System.out.println("\n" + objectSupplier.title());

            for (SortImpl sortImplementation : sortImplementations) {
                Object[] sortInput = objectSupplier.getCopy();

                final long startNanos = System.nanoTime();
                sortImplementation.sort(sortInput);
                final long stopNanos = System.nanoTime();

                final long durNanos = stopNanos - startNanos;
                final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
                final String durFormatted = LongFormatter.formatUnderscore(durMillis);
                System.out.println(durFormatted + "," + sortImplementation.title);
            }
        }
    }

    static SortImpl[] getSortImplementations() {
        return new SortImpl[]{
                new SortImpl("DummySort") {
                    @Override
                    public void sort(Object[] a) {
                        DummySort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("TimSort") {
                    @Override
                    public void sort(Object[] a) {
                        TimSort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("MergeSort") {
                    @Override
                    public void sort(Object[] a) {
                        MergeSort.legacyMergeSort(a, NaturalOrder.INSTANCE);
                    }
                }
        };
    }

    /**
     * The returned ObjectSupplier objects are wrapped by Supplier objects.
     * This way they are lazily created on their first access.
     * This saves memory if we work with large lists.
     */
    static List<Supplier<ObjectSupplier>> getSortInputSuppliers() {
        return Arrays.asList(
                // Three different random lists.
                RandomIntegers::new,
                RandomIntegers::new,
                RandomIntegers::new,
                // One descending list.
                DescendingIntegers::new
        );
    }
}
